# Generated by roxygen2: do not edit by hand

export(alphaScreening)
export(alphaTesting)
export(msharpe)
export(msharpeScreening)
export(msharpeTesting)
export(sharpe)
export(sharpeScreening)
export(sharpeTesting)
import(compiler)
import(lmtest)
import(sandwich)
importFrom(compiler,cmpfun)
importFrom(parallel,clusterApplyLB)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(stats,qnorm)
