#' @name hfdata
#' @title Hedge fund data
#' @description The matrix \code{hfdata} contains randomized and modified monthly returns of hedge funds.
#' @source HFR database
#' @docType data
#' @keywords datasets
#' @usage data(hfdata)
#' @format A matrix of size 60 x 100, containing 60 monthly net returns for 100 hedge funds. Returns have been modified.
NULL